# Site

This is the main repository with the source files for UltraClub's website.

Database creation is *not automated* and should be handled separately.

To redefine the general configurations found in **settings.json** create an
**env.txt** file on the root and within it choose an environment name, after
that any settings present on **settings.(environment-name).json** will override
the default ones in **settings.json**. Use this to fill in environment specific
settings such as database connection info.


### Structure
Within the **projects** folder you'll find the folders for project specific files
**_shared**, **app**, **main** and **panel**.

#### _shared
Files there are accessible from any of the projects, **_shared/actions** can be
called on any of them, **_shared/classes** can be used in any of them,
**_shared/dynamic** and **_shared/static** can be accessed through the public URL
in any of the projects as if they were in the projects respective folders, but should
there exist a file with the same name in the project that one will take priority.

#### main
It's the main website, accessible at http://ultraclub.club.

#### app
Is the user area.

#### panel
Is the admin panel.

### .htaccess behavior
#### {project-domain}/{page}
Will render a view **projects/{project}/pages/{page}.php** within the context of
**projects/{project}/index.php**

#### {project-domain}/do/{controller}.{action}
Will invoke the action in the respective controller, in case an action is not
determined it presumes it to have the same name as the controller.

Controllers are looked up in this order:
**projects/{project}/actions/{controller}.php**,
**projects/_shared/actions/{controller}.php**,
**projects/actions/{controller}.php**

#### {project-domain}/{any-path}
It'll look for files in this order;
**projects/{project}/{any-path}**,
**projects/_shared/{any-path}**,
**{any-path}** (as long as it's within the static folder)


### Backend
A custom framework is used, rarely if at all direct queries should be used
instead giving preference to using the ORM in place.

The database is MySQL and can be procedurally generated by the ORM albeit
there's no automated process to do so at the moment, for now, importing a database
dump should suffice.

### Front end
Standard css and js (no sass/babel/...).

Scripts have a dependency chain documented within a **Dependency.json** file and are
included using PHP that uses it to make sure all dependencies have been included
before and to avoid including a script twice by accident. It's simple as:

> \<?=ScriptDependency::Current()->IncludeScript('entities/User')?>

Also used is Vue.js, components although stored in .vue files are parsed by PHP
and included inline, that's sub-optimal and compiling them into a cacheable .js
file is expected down the road. To do it just use

> \<?=VueParser::Inline('FileUploader')?>