<?PHP
abstract class BaseEnum {
    private static $constCache = [];

    public static function getConstants() {
        $c = get_called_class();
        if (!isset(self::$constCache[$c])) {
            $reflect = new ReflectionClass(get_called_class());
            self::$constCache[$c] = $reflect->getConstants();
        }

        return self::$constCache[$c];
    }

    public static function isValidName($name, $strict = false) {
        $constants = self::getConstants();

        if ($strict) {
            return array_key_exists($name, $constants);
        }

        $keys = array_map('strtolower', array_keys($constants));
        return in_array(strtolower($name), $keys);
    }

    public static function isValidValue($value) {
        $values = array_values(self::getConstants());
        return in_array($value, $values, $strict = true);
    }

    public static function ValueFromName($name, $def = null) {
        $constants = self::getConstants();

        if(array_key_exists($name, $constants)){
            return $constants[$name];
        }

        return $def;
    }
}