<?PHP

/**
 * @property string Code
 * @property integer TypeId
 * @property integer OwnerId
 *
 * @property string Title
 * @property string Brand
 * @property string Model
 * @property string Description
 *
 * @property integer Capacity In people
 * @property float CarryCapacity In kg
 *
 * @property integer DefaultCost
 * @property integer DefaultCostMeasurement
 * @property integer VariableCost
 * @property integer VariableCostMeasurement
 *
 * @property string Observations
 * @property string Home
 *
 * @property integer PlansAllowed
 * @property boolean Hidden
 * 
 * @property GoodType Type
 * @property User Owner
 **/
class Good extends BaseModel {
	use MagicEntity;
	
	public static $Identifier = ["Id"];
	public static $databaseTable = 'good';


	private $_imgUrl;
	public function getImagesUrl(){
		if (!$this->_imgUrl){
			$this->_imgUrl = GetDynamicUrl('main').'item_images/'.$this->Id.'/';
		}

		return $this->_imgUrl;
	}
	public function getImagesDir(){
		$dir = GetDynamicDirectory('main').'item_images/'.$this->Id.'/';
		if (!file_exists($dir)){
			FileSystem::CreateDirectory($dir);
		}

		return $dir;
	}
	public static function getTempImageUrl(){
		return GetDynamicUrl('main').'item_images/temp/';
	}
	public static function getTempImageDir(){
		return GetDynamicDirectory('main').'item_images/temp/';
	}

	public function getImages($includeCacheParameter=true){
		$dir = $this->getImagesDir();

		return array_map(function ($e) use($dir, $includeCacheParameter){
			return str_ireplace($dir, '', $e).($includeCacheParameter ? '?t='.filemtime($e) : '');
		}, glob($dir.'*'));
	}

	public $_imagesToInclude = [];
	public $_imagesToRemove = [];
	
	public function Save($insert = null) {
		$id = parent::Save($insert);

		if ($this->Id){
			$dir = $this->getImagesDir();
			$tempDir = $this->getTempImageDir();


			foreach ($this->_imagesToRemove as $img){
				$img = explode('?', $img)[0];

				if (file_exists($tempDir.$img)) unlink($tempDir.$img);
				if (file_exists($dir.$img)) unlink($dir.$img);
			}

			foreach ($this->_imagesToInclude as $img){
				if (file_exists($tempDir . $img)){
					rename($tempDir.$img, $dir.$img);
				}
			}

			if (count($this->_imagesToInclude) || count($this->_imagesToRemove)){
				$images = $this->getImages(false);
				foreach ($images as $i=>$image){
					rename($dir.$image, $dir.time().'_'.$i);
				}

				clearstatcache();
				$images = $this->getImages(false);
				foreach ($images as $i=>$image){
					rename($dir.$image, $dir.$i);
				}
			}

			$this->_imagesToInclude = [];
			$this->_imagesToRemove = [];
		}

		return $id;
	}

	public function Delete() {
		$result = parent::Delete();

		$dir = $this->getImagesDir();

		$images = $this->getImages(false);
		foreach ($images as $i=>$image){
			unlink($dir.$image);
		}
		
		FileSystem::RemoveDir($dir);

		return $result;
	}
}
Good::SetRelationships([
	'Owner'=> (new RelationshipManyToOne('User', 'OwnerId'))->Magic(),
	'Type'=> (new RelationshipManyToOne('GoodType', 'TypeId'))->Autoload(true)->Magic(),
	'Operators'=> (new RelationshipOneToMany('GoodOperator', 'Id', 'GoodId'))->Autoload(true),
]);

Good::SetFields([
	(new IntegerField('Id'))
		->SetDatabaseDescriptor('int', 11)->AutoIncrement(true)->PrimaryKey(),

	(new NotNullField('Code', ''))
		->SetDatabaseDescriptor('varchar', 256),

	(new NotNullField('TypeId', ''))
		->SetDatabaseDescriptor('int', 11),

	(new NotNullField('OwnerId', ''))
		->SetDatabaseDescriptor('int', 11),

	(new NotNullField('Title', ''))
		->SetDatabaseDescriptor('varchar', 512),

	(new NotNullField('Brand', ''))
		->SetDatabaseDescriptor('varchar', 256),

	(new NotNullField('Model', ''))
		->SetDatabaseDescriptor('varchar', 128),

	(new NotNullField('Description', ''))
		->SetDatabaseDescriptor('varchar', 512),

	(new IntegerField('Capacity'))
		->SetDatabaseDescriptor('int', 4),

	(new IntegerField('CarryCapacity'))
		->SetDatabaseDescriptor('float'),

	(new IntegerField('DefaultCost'))
		->SetDatabaseDescriptor('int', 12),

	(new IntegerField('DefaultCostMeasurement'))
		->SetDatabaseDescriptor('int', 3),

	(new IntegerField('VariableCost'))
		->SetDatabaseDescriptor('int', 12),

	(new IntegerField('VariableCostMeasurement'))
		->SetDatabaseDescriptor('int', 3),

	(new NotNullField('Observations', ''))
		->SetDatabaseDescriptor('varchar', 512),

	(new NotNullField('Home', ''))
		->SetDatabaseDescriptor('varchar', 512),

	(new IntegerField('PlansAllowed'))
		->SetDatabaseDescriptor('int', 4),

	(new BooleanField('Hidden'))
		->SetDatabaseDescriptor('tinyint', 1),

	(new ComputedField('ImagesUrl', 'getImagesUrl'))
		->InDatabase(false),

	(new ComputedField('Images', 'getImages'))
		->InDatabase(false),

	(new Field('_imagesToInclude'))
		->InDatabase(false),

	(new Field('_imagesToRemove'))
		->InDatabase(false),
]);