<?PHP

/**
 * @property integer Id
 * @property integer Index
 * @property string Name
 * @property string PagseguroId
 * @property float MonthlyCost
 * @property integer MaxDependents
 * @property float CreditsBonus
 * @property integer CreditsExpiryInMonths
 * @property integer MinimumMembershipTime
 * @property boolean Active
 **/
class Plan extends BaseModel {
	use MagicEntity;
	public static $databaseTable = 'plan';

	private static $pointsRatio = 1.618*2;

	public function GetPagSeguroPlan(){
		if (!$this->PagseguroId) {

			$x = PagSeguroHandler::json('/pre-approvals/request', 'POST', [
				'reference'   => $this->Id,
				'preApproval' => [
					'name'             => $this->Name,
					'charge'           => 'AUTO',
					'period'           => 'MONTHLY',
					'amountPerPayment' => $this->MonthlyCost . '.00',
					'expiration'       => [
						'value' => 2,
						'unit'  => 'YEARS'
					]
				]
			]);

			if ($x['code']) {
				$this->PagseguroId = $x['code'];
				$this->Save();
			}
		}

		if ($this->PagseguroId){
			return $this->PagseguroId;
		}else{
			return false;
		}
	}

	public function CreditsPerMonth(){
		return $this->CalculateCreditsFromAmount($this->MonthlyCost);
	}

	public function CalculateCreditsFromAmount($amount){
		return floor(self::$pointsRatio * $amount * (1+($this->CreditsBonus/100)));
	}
}

Plan::SetFields([
	(new IntegerField('Id'))
		->SetDatabaseDescriptor('int', 11)->AutoIncrement(true)->PrimaryKey(),

	(new IntegerField('Index'))
		->SetDatabaseDescriptor('smallint', 3),

	(new NotNullField('Name', ''))
		->SetDatabaseDescriptor('varchar', 128),

	(new NotNullField('PagseguroId', ''))
		->SetDatabaseDescriptor('varchar', 32),

	(new FloatField('MonthlyCost'))
		->SetDatabaseDescriptor('float'),

	(new ComputedField('CreditsPerMonth'))
		->SetDatabaseDescriptor('smallint', 6),

	(new IntegerField('MaxDependents'))
		->SetDatabaseDescriptor('tinyint', 3),

	(new FloatField('CreditsBonus'))
		->SetDatabaseDescriptor('float'),

	(new IntegerField('CreditsExpiryInMonths'))
		->SetDatabaseDescriptor('smallint', 3),

	(new IntegerField('MinimumMembershipTime'))
		->SetDatabaseDescriptor('smallint', 3),

	(new BooleanField('Active'))
		->SetDatabaseDescriptor('tinyint', 1),
]);