<?PHP

/**
 * @property integer Id
 * @property integer RenterId
 * @property integer GoodId
 * @property integer StartDateTime
 * @property integer EndDateTime
 * @property integer PickUpAddressId
 * @property integer FixedCredits
 * @property integer VariableCredits
 * @property integer ExtraCredits
 * @property string  Observations
 * @property array   Meta
 *
 * @property integer CheckInTime
 * @property string  CheckInObservations
 * @property integer CheckOutTime
 * @property string  CheckOutObservations
 *
 * @property integer Status
 *
 * @property boolean ManualPayment
 * @property string  PaymentCode
 * @property string  PaymentUserId
 *
 *
 * @property Good    Good
 * @property User    Renter
 * @property Address PickupAddress
 **/
class Reservation extends BaseModel {
	use MagicEntity;
	public static $databaseTable = 'reservation';


	public function getCost() {
		return $this->FixedCredits + $this->VariableCredits;
	}

	public function getStatusString(){
	    return ReservationStatus::$strings[$this->Status];
    }
}

Reservation::SetRelationships([
	'Good'          => (new RelationshipManyToOne('Good', 'GoodId'))->Autoload(true)->Magic()->OnDelete('CASCADE'),
	'Renter'        => (new RelationshipManyToOne('User', 'RenterId'))->Autoload(true)->Magic()->OnDelete('CASCADE'),
	'PickupAddress' => (new RelationshipManyToOne('Address', 'PickupAddressId'))->Autoload(true)->Magic()->OnDelete('CASCADE')
	                                                                            ->SaveParentToo()->DeleteParentToo(),
]);

Reservation::SetFields([
	(new IntegerField('Id'))
		->SetDatabaseDescriptor('int', 11)->AutoIncrement(true)->PrimaryKey(),

	(new IntegerField('GoodId'))
		->SetDatabaseDescriptor('int', 11),

	(new IntegerField('RenterId'))
		->SetDatabaseDescriptor('int', 11),

	(new IntegerField('StartDateTime'))
		->SetDatabaseDescriptor('int', 11),

	(new IntegerField('EndDateTime'))
		->SetDatabaseDescriptor('int', 11),

	(new IntegerField('PickupAddressId'))
		->SetDatabaseDescriptor('int', 11)->Null(true),

	(new IntegerField('FixedCredits'))
		->SetDatabaseDescriptor('int', 11),

	(new IntegerField('VariableCredits'))
		->SetDatabaseDescriptor('int', 11),

	(new IntegerField('ExtraCredits'))
		->SetDatabaseDescriptor('int', 11),

	(new NotNullField('Observations', ''))
		->SetDatabaseDescriptor('text'),

	(new JsonField('Meta'))
		->SetDatabaseDescriptor('text'),


	(new NullableIntegerField('CheckInTime'))
		->SetDatabaseDescriptor('int', 11)->Null(true),

	(new NotNullField('CheckInObservations', ''))
		->SetDatabaseDescriptor('text'),

	(new NullableIntegerField('CheckOutTime'))
		->SetDatabaseDescriptor('int', 11)->Null(true),

	(new NotNullField('CheckOutObservations', ''))
		->SetDatabaseDescriptor('text'),

	(new IntegerField('Status'))
		->SetDatabaseDescriptor('smallint', 4),


	(new BooleanField('ManualPayment'))
		->SetDatabaseDescriptor('tinyint', 1),

	(new NotNullField('PaymentCode', ''))
		->SetDatabaseDescriptor('text'),

	(new NullableIntegerField('PaymentUserId'))
		->SetDatabaseDescriptor('int', 11)
		->Null(true),
]);