<?PHP

/**
 * @property integer Id
 * @property integer UserId
 * @property integer PlanId
 * @property string  PagseguroId
 *
 * @property integer StartDate
 * @property integer EndDate
 * @property integer Status
 * @property float   MonthlyCost
 * @property integer CreditsPerMonth
 *
 * @property User    User
 * @property Plan    Plan
 **/
class Subscription extends BaseModel {
	use MagicEntity;
	public static $databaseTable = 'subscription';


	/**
	 * @param $token  string
	 * @param $holder User
	 * @param $hash   string
	 *
	 * @return bool
	 * @throws Exception
	 */
	function Create($token, $holder, $hash) {
		if (!$this->Plan->PagseguroId) {
			if (!$this->Plan->GetPagSeguroPlan()) {
				throw new Exception('Failed to create PagSeguro plan');
			}
		}

		if (!$this->Id) {
			$this->Save();
		}

		if (!PagSeguroHandler::$sandbox) {
			$email = $this->User->Email;
		} else {
			$email = explode('@', $this->User->Email)[0] . '@sandbox.pagseguro.com.br';
		}


		$data = [
			'plan'          => $this->Plan->PagseguroId,
			'reference'     => $this->Id,
			'sender'        => [
				'name'      => $this->User->FullName,
				'email'     => $email,
				'hash'      => $hash,
				'phone'     => [
					'areaCode' => $this->User->TelephoneAreaCode(),
					'number'   => $this->User->TelephoneNumber()
				],
				'address'   => [
					'street'     => $this->User->Address->Street,
					'number'     => $this->User->Address->Number,
					'complement' => $this->User->Address->NumberComplement,
					'district'   => $this->User->Address->Neighborhood,
					'city'       => $this->User->Address->City,
					'state'      => $this->User->Address->State,
					'country'    => 'BRA',
					'postalCode' => $this->User->Address->PostalCode
				],
				'documents' => [
					[
						'type'  => 'CPF',
						'value' => $this->User->CPFNumber()
					]
				]
			],
			'paymentMethod' => [
				'type'       => 'CREDITCARD',
				'creditCard' => [
					'token'  => $token,
					'holder' => [
						'name'           => $holder->FullName,
						'birthDate'      => date('d/m/Y', $holder->Birthday),
						'documents'      => [
							[
								'type'  => 'CPF',
								'value' => $holder->CPFNumber()
							]
						],
						'phone'          => [
							'areaCode' => $holder->TelephoneAreaCode(),
							'number'   => $holder->TelephoneNumber()
						],
						'billingAddress' => [
							'street'     => $holder->Address->Street,
							'number'     => $holder->Address->Number,
							'complement' => $holder->Address->NumberComplement,
							'district'   => $holder->Address->Neighborhood,
							'city'       => $holder->Address->City,
							'state'      => $holder->Address->State,
							'country'    => 'BRA',
							'postalCode' => $holder->Address->PostalCode
						],
					]
				]
			]
		];

		$result = PagSeguroHandler::json('/pre-approvals', 'POST', $data);

		if ($result['code']) {
			$this->PagseguroId = $result['code'];
			$this->Save();

			return true;
		}

		return false;
	}


	/**
	 * @param $amount int
	 * @param $token  string
	 * @param $holder User
	 * @param $hash   string
	 *
	 * @return bool
	 * @throws Exception
	 */
	function BuyCredits($amount, $token, $holder, $hash) {
		if (!PagSeguroHandler::$sandbox) {
			$email = $this->User->Email;
		} else {
			$email = explode('@', $this->User->Email)[0] . '@sandbox.pagseguro.com.br';
		}

		$credits = $this->Plan->CalculateCreditsFromAmount($amount);

		$data = [
			'mode'      => 'default',
			'currency'  => 'BRL',
			'notificationURL' => GetActionUrl('Pagseguro', 'Notify'),
			'reference' => $this->User->Id,
			'sender'    => [
				'name'      => $this->User->FullName,
				'email'     => $email,
				'hash'      => $hash,
				'phone'     => [
					'areaCode' => $this->User->TelephoneAreaCode(),
					'number'   => $this->User->TelephoneNumber()
				],
				'address'   => [
					'street'     => $this->User->Address->Street,
					'number'     => $this->User->Address->Number,
					'complement' => $this->User->Address->NumberComplement,
					'district'   => $this->User->Address->Neighborhood,
					'city'       => $this->User->Address->City,
					'state'      => $this->User->Address->State,
					'country'    => 'BRA',
					'postalCode' => $this->User->Address->PostalCode
				],
				'documents' => [
					'document' => [
						'type'  => 'CPF',
						'value' => $this->User->CPFNumber()
					]
				]
			],

			'shipping'=>[
				'addressRequired'=>false
			],

			'items' => [
				'item'=>[
					'id'          => 'CREX',
					'description' => "$credits pontos",
					'amount'      => $amount . '.00',
					'quantity'    => 1
				]
			],

			'method' => 'creditCard',

			'creditCard' => [
				'token'          => $token,
				'capture'        => true,
				'holder'         => [
					'name'      => $holder->FullName,
					'birthDate' => date('d/m/Y', $holder->Birthday),
					'documents' => [
						'document' => [
							'type'  => 'CPF',
							'value' => $holder->CPFNumber()
						]
					],
					'phone'     => [
						'areaCode' => $holder->TelephoneAreaCode(),
						'number'   => $holder->TelephoneNumber()
					]
				],
				'billingAddress' => [
					'street'     => $holder->Address->Street,
					'number'     => $holder->Address->Number,
					'complement' => $holder->Address->NumberComplement,
					'district'   => $holder->Address->Neighborhood,
					'city'       => $holder->Address->City,
					'state'      => $holder->Address->State,
					'country'    => 'BRA',
					'postalCode' => $holder->Address->PostalCode
				],
				'installment'=>[
					'quantity'=>1,
					'value'=>$amount . '.00'
				]
			]
		];

		$data = PagSeguroHandler::encodeXml($data, 'payment');

		$result = PagSeguroHandler::xml('/transactions', 'POST', $data, true);

		if ($result['status']) {
			return true;
		}

		return false;
	}

	function End() {
		if ($this->PagseguroId) {
			$result = PagSeguroHandler::json('/pre-approvals/' . $this->PagseguroId . '/cancel/', 'PUT');
		}

		$this->EndDate = time();
		$this->Save();
	}
}

Subscription::SetRelationships([
	'Plan' => (new RelationshipManyToOne('Plan', 'PlanId'))->Autoload(true)->Magic()->OnDelete('SET NULL'),
	'User' => (new RelationshipManyToOne('User', 'UserId'))->Magic()->OnDelete('CASCADE')
]);

Subscription::SetFields([
	(new IntegerField('Id'))
		->SetDatabaseDescriptor('int', 11)->AutoIncrement(true)->PrimaryKey(),

	(new IntegerField('UserId'))
		->SetDatabaseDescriptor('int', 11),

	(new NullableIntegerField('PlanId'))
		->SetDatabaseDescriptor('int', 11)
		->Null(true),

	(new NotNullField('PagseguroId', ''))
		->SetDatabaseDescriptor('varchar', 32),

	(new IntegerField('StartDate'))
		->SetDatabaseDescriptor('int', 11),

	(new NullableIntegerField('EndDate'))
		->SetDatabaseDescriptor('int', 11)
		->Null(true),

	(new IntegerField('Status'))
		->SetDatabaseDescriptor('smallint', 3),

	(new IntegerField('MonthlyCost'))
		->SetDatabaseDescriptor('float'),

	(new IntegerField('CreditsPerMonth'))
		->SetDatabaseDescriptor('smallint', 6)
]);