<?PHP

/**
 * @property integer Id
 * @property string Email
 * @property string Password
 * @property integer Status
 * @property integer Role
 * @property integer CreationDate
 * @property string FullName
 * @property string DisplayName
 * @property integer Gender
 * @property string CPF
 * @property string Telephone
 * @property integer Birthday
 * @property integer CurrentSubscriptionId
 * @property integer Balance
 * @property string FacebookId
 * @property boolean ReceiveEmailMarketing
 * @property integer AddressId
 * @property Address Address
 *
 * @property integer ApprovalDate
 * @property integer ApprovalUserId
 * @property integer LastAccessDate
 *
 * Ships
 * @property Subscription CurrentSubscription
 * @property Subscription[] Subscriptions
 **/
class User extends BaseModel {
	use MagicEntity;
	public static $databaseTable = 'user';

	function __construct($id=null) {
		$this->Avatar = new DynamicFile(DIR_DYNAMIC.'/avatar/', function($obj){
			return $obj->Id;
		}, '_shared', $this);

		$this->AvatarThumb = new DynamicFile(DIR_DYNAMIC.'/avatar/', function($obj){
			return $obj->Id.'_square';
		}, '_shared', $this);

		$this->CreationDate = time();
		
		parent::__construct($id);
	}

	public function Delete() {
		$result = parent::Delete(); // TODO: Change the autogenerated stub

		if ($this->HasAvatar()){
			unlink($this->Avatar->GetPath());
			unlink($this->AvatarThumb->GetPath());
		}

		return $result;
	}

	////// PASSWORD
	public function HashPassword($pass){
		return hash('sha256', $pass.'_'.$this->CreationDate);
	}
	public function SetPassword($pass=null){
		if ($pass){
			$oPass = $this->Password;

			$this->Password = $this->HashPassword($pass);

			if (strlen($oPass) == $this->Password){
				$this->SendMail('Senha alterada',
					EmailTemplate::RenderContent('PasswordChange.html', ['user'=>$this]),
					'password-change-alert');
			}
		}
	}

	////// AUTH
	public function LoginAs(){
		$_SESSION['user'] = $this->Id;
	}

	public function CookieHash($time){ return hash('sha256', $this->CreationDate . $this->Id . $time); }
	public function SetCookies() {
		$_SESSION['remember'] = 1;

		$time = time();

		$status = true;
		$status = $status && setcookie('uid', $this->Id, $time+60*60*24*30, '/');
		$status = $status && setcookie('time', $time, $time+60*60*24*30, '/');
		$status = $status && setcookie('hash', $this->CookieHash($time), $time+60*60*24*30, '/');
	}
	public static function UnsetCookies() {
		$_SESSION['remember'] = 0;

		unset($_COOKIE['uid'], $_COOKIE['time'], $_COOKIE['hash']);

		setcookie('uid', 0, 0, '/');
		setcookie('time', 0, 0, '/');
		setcookie('hash', 0, 0, '/');
	}

	public static function Logout(){
		unset($_SESSION['user']);
		$_SESSION['logout'] = 1;
	}

	public static function LoggedUser(){
		if (isset($_SESSION['logout']) && $_SESSION['logout']){
			self::UnsetCookies();
			$_SESSION['logout'] = 0;
		}

		if (!isset($_SESSION['user']) && isset($_COOKIE['uid'])){
			$user = new User($_COOKIE['uid']);

			if ($_COOKIE['hash'] == $user->CookieHash($_COOKIE['time'])){
				$user->LoginAs();
				return $user;
			}
		}


		if (isset($_SESSION['user'])) {
			$user = new User($_SESSION['user']);
			if ($user->Exists){
				if (isset($_SESSION['remember']) && $_SESSION['remember']){
					$user->SetCookies();
				}
				return $user;
			}else{
				self::Logout();
			}
		}
		return null;
	}
	public static function LoggedUserOrDie(){
		$user = self::LoggedUser();
		if ($user){
			return $user;
		}else{
			die();
		}
	}
	public static function LoggedUserOrHome(){
		$user = self::LoggedUser();
		if ($user){
			return $user;
		}else{
			Redirect(GetProjectUrl());
			die();
		}
	}

    public function GetConfirmationHash(){
        return hash('sha256', $this->Id.'-'.$this->CreationDate.'-'.$this->CPF);
    }

    public function GetPasswordResetHash($time){
        return hash('sha256', $this->GetConfirmationHash().$this->Password.$time.'password');
    }
	
	////// IMAGE
	/** @var DynamicFile */
	public $Avatar;
	/** @var DynamicFile */
	public $AvatarThumb;

	public function GetImageFromFacebook(){
		$fb = FacebookHandler::getInstance();
		$r = $fb->api('/'.$this->FacebookId.'/picture/?width=720&redirect=false', 'GET');

		if (isset($r['url'])){
			FileSystem::Download($r['url'], $this->Avatar->GetPath());
			$this->AvatarUpdateThumbnail();
		}
	}
	public function HasAvatar(){
		return $this->Avatar->Exists(true);
	}
	public function AvatarUpdateThumbnail(){
		if (!$this->HasAvatar()){
			return;
		}

		ImageHandler::CreateThumbNail(
			$this->Avatar->GetPath(),
			'',
			$this->AvatarThumb->GetPath(),
			200,
			200,
			true,
			true);
	}

	///// Balance
	public function BalanceOperation($amount, $description){
		$bo = new BalanceOperation();
		$bo->UserId = $this->Id;
		$bo->Type = $amount > 0 ? BalanceOperationType::CreditOrder : BalanceOperationType::DebitOrder;
		$bo->Amount = abs($amount);
		$bo->Description = $description;
		$bo->Date = time();
		$bo->Save();

		if ($bo->Id){
			$this->Balance += $amount;
			$this->Save();
		}

        // Notify user via email
        $this->SendMail('Operação de '.$bo->getOperationString().' no seu saldo',
            EmailTemplate::RenderContent('BalanceChange.html', ['user'=>$this, 'bo'=> $bo], 'app'),
            'ultra-balance-change');

		return $bo;
	}
	
	///// Sub
	/**
	 * @param Plan $newPlan
	 *
	 * @return Subscription
	 */
	public function SubscriptionChange($newPlan){
		/** @var Subscription[] $activeSubs */
		$activeSubs = Subscription::Select([
			'UserId'=>$this->Id,
			new Filter('EndDate', 'NULL', 'IS', 'OR', false)
		]);

		foreach ($activeSubs as $activeSub){
			$activeSub->End();
		}
		
		$newSub = new Subscription();
		$newSub->UserId = $this->Id;
		$newSub->PlanId = $newPlan->Id;
		$newSub->Plan = $newPlan;
		$newSub->StartDate = time();
		$newSub->EndDate = null;
		$newSub->Status = 1;
		$newSub->MonthlyCost = $newPlan->MonthlyCost;
		$newSub->CreditsPerMonth = $newPlan->CreditsPerMonth;
		$newSub->Save();

		if ($newSub->Id){
			$this->CurrentSubscription = $newSub;
			$this->CurrentSubscriptionId = $newSub->Id;
			$this->Save();
		}

		return $newSub;
	}


	public function SendMail($subject, $message, $tag=null){
		$mail = new Email();
		$mail->Sender = 'noreply@ultraclub.club';
		$mail->SenderName = 'Ultra Club (não responder)';
		$mail->Destination = $this->Email;
		$mail->DestinationName = $this->DisplayName;
		$mail->Subject = $subject;
		$mail->Message = $message;

		return $mail->Mandrill($tag);
	}

	public function HasRole($role){
		return $this->Role & $role ? true : false;
	}


	public function TelephoneAreaCode(){
		$numbersOnlyPhone = preg_replace( '/[^0-9]/', '', $this->Telephone);
		$len = strlen($numbersOnlyPhone);

		if ($len == 10 || $len == 11){
			return substr($numbersOnlyPhone, 0, 2);
		}else if ($len == 12 || $len == 13){
			return substr($numbersOnlyPhone, 2, 2);
		}if ($len == 14){
			return substr($numbersOnlyPhone, 3, 2);
		}

		return false;
	}
	public function TelephoneNumber(){
		$numbersOnlyPhone = preg_replace( '/[^0-9]/', '', $this->Telephone);
		$len = strlen($numbersOnlyPhone);

		if ($len == 10 || $len == 11){
			return substr($numbersOnlyPhone, 2);
		}else if ($len == 12 || $len == 13){
			return substr($numbersOnlyPhone, 4);
		}if ($len == 14){
			return substr($numbersOnlyPhone, 5);
		}

		return false;
	}
	public function CPFNumber(){
		return preg_replace('/[^0-9]/', '', $this->CPF);
	}
}

User::SetRelationships([
	'Address'=> (new RelationshipManyToOne('Address', 'AddressId'))->Autoload(true)->SaveParentToo()->DeleteParentToo()->Magic()->OnDelete('SET NULL'),
	'ApprovalUser'=> (new RelationshipManyToOne('User', 'ApprovalUserId'))->Autoload(true)->Magic()->OnDelete('SET NULL'),
	'CurrentSubscription'=> (new RelationshipManyToOne('Subscription', 'CurrentSubscriptionId'))->Autoload(true)->Magic()->OnDelete('SET NULL'),
	'Subscriptions'=> (new RelationshipOneToMany('Subscription', 'Id', 'UserId', true))->OnDelete('CASCADE'),
	'BalanceOperations'=> (new RelationshipOneToMany('BalanceOperation', 'Id', 'UserId', true))->OnDelete('CASCADE'),
]);

User::SetFields([
	(new IntegerField('Id'))
		->SetDatabaseDescriptor('int', 11)->AutoIncrement(true)->PrimaryKey(),

	(new NotNullField('Email', ''))
		->SetDatabaseDescriptor('varchar', 512),

	(new NotNullField('Password', ''))
		->SetDatabaseDescriptor('varchar', 64)
		->Sensitive(true),

	(new IntegerField('Status'))
		->SetDatabaseDescriptor('smallint', 6),

	(new IntegerField('Role'))
		->SetDatabaseDescriptor('smallint', 6),

	(new IntegerField('CreationDate'))
		->SetDatabaseDescriptor('int', 11)
		->Sensitive(true),

	(new NotNullField('FullName', ''))
		->SetDatabaseDescriptor('varchar', 512),

	(new NotNullField('DisplayName', ''))
		->SetDatabaseDescriptor('varchar', 128),

	(new IntegerField('Gender'))
		->SetDatabaseDescriptor('tinyint', 2),

	(new NotNullField('CPF', ''))
		->SetDatabaseDescriptor('varchar', 14)
		->Sensitive(true),

	(new NotNullField('Telephone', ''))
		->SetDatabaseDescriptor('varchar', 32)
		->Sensitive(true),

	(new NullableIntegerField('Birthday'))
		->SetDatabaseDescriptor('int', 11)
		->Sensitive(true)
		->Null(true),

	(new NullableIntegerField('CurrentSubscriptionId'))
		->SetDatabaseDescriptor('int', 11)
		->Null(true),

	(new IntegerField('Balance'))
		->SetDatabaseDescriptor('int', 11),

	(new NotNullField('FacebookId', 0))
		->SetDatabaseDescriptor('bigint', 20)
		->Unsigned(true)
		->Sensitive(true),

	(new ComputedField('HasAvatar'))
		->InDatabase(false),

	(new BooleanField('ReceiveEmailMarketing'))
		->SetDatabaseDescriptor('tinyint', 1),

	(new NullableIntegerField('AddressId'))
		->SetDatabaseDescriptor('int', 11)
		->Null(true),


	(new IntegerField('ApprovalDate'))
		->SetDatabaseDescriptor('int', 11),

	(new NullableIntegerField('ApprovalUserId'))
		->SetDatabaseDescriptor('int', 11)
		->Null(true),

	(new IntegerField('LastAccessDate'))
		->SetDatabaseDescriptor('int', 11),


	(new ComputedField('SetPassword'))
		->InDatabase(false)
]);