/*
 * Description: A script
 * Version: 0.1
 * Author: Daniel Andrade
 * Date: 26/01/2017
 * This code may not be reused without proper permission from its creator.
 */

/**
 * @property {number} Id
 * @property {string} Email
 * @property {string} Password
 * @property {number} Status
 * @property {number} Role
 * @property {string} FullName
 * @property {string} DisplayName
 * @property {number} Gender
 * @property {string} CPF
 * @property {string} Telephone
 * @property {Date}   Birthday
 * @property {number} CurrentSubscriptionId
 * @property {number} Balance
 * @property {boolean} ReceiveEmailMarketing
 * @property {boolean} HasAvatar
 *
 * @property {Address} Address
 * @property {Subscription} CurrentSubscription
 *
 * @extends ORM.Entity
 * @constructor
 */
function User(){
    this.Parent(null, arguments, User);

    if (this.Address == null){
        this.Address = new Address();
    }
}
User.inherit(Entity);
User.Attributes = [
    new Entity.Attributes.Integer('Id'),
    new Entity.Attributes.String('FullName'),
    new Entity.Attributes.String('DisplayName'),
    new Entity.Attributes.String('Email'),
    new Entity.Attributes.String('Password'),
    new Entity.Attributes.String('SetPassword'),
    new Entity.Attributes.Integer('Status'),
    new Entity.Attributes.Integer('Role'),
    new Entity.Attributes.Integer('Gender'),
    new Entity.Attributes.String('CPF'),
    new Entity.Attributes.String('Telephone'),
    new Entity.Attributes.UnixTimestamp('Birthday', true),
    new Entity.Attributes.Integer('CurrentSubscriptionId'),
    new Entity.Attributes.Integer('Balance'),
    new Entity.Attributes.Boolean('ReceiveEmailMarketing'),
    new Entity.Attributes.Boolean('HasAvatar'),
    new Entity.Attributes.Entity('Address', false),
    new Entity.Attributes.Entity('CurrentSubscription', false),
    new Entity.Attributes.EntityList('Subscriptions', false),
    new Entity.Attributes.UnixTimestamp('ApprovalDate'),
    new Entity.Attributes.UnixTimestamp('LastAccessDate'),
    new Entity.Attributes.Entity('ApprovalUser', false),
];
Entity.ClassMap.Register(User);

User.prototype.avatarUrl = function(){
    var url = '';
    if (Session && Session.URL) url = Session.URL;

    if (!this.HasAvatar) return url + 'static/images/user-no-pic.png';


    url += 'dynamic/avatar/'+ this.Id+"_square";

    return url;
};
User.prototype.isCpfValid = function(){
    var cpf = this.CPF.replace(/[^\d]+/g,'');
    
    if(cpf == '') return false;
    // Elimina CPFs invalidos conhecidos
    if (cpf.length != 11 ||
        cpf == "00000000000" ||
        cpf == "11111111111" ||
        cpf == "22222222222" ||
        cpf == "33333333333" ||
        cpf == "44444444444" ||
        cpf == "55555555555" ||
        cpf == "66666666666" ||
        cpf == "77777777777" ||
        cpf == "88888888888" ||
        cpf == "99999999999")
        return false;
    // Valida 1o digito
    var add = 0;
    for (var i=0; i < 9; i ++)
        add += parseInt(cpf.charAt(i)) * (10 - i);
    var rev = 11 - (add % 11);
    if (rev == 10 || rev == 11)
        rev = 0;
    if (rev != parseInt(cpf.charAt(9)))
        return false;
    // Valida 2o digito
    add = 0;
    for (var i = 0; i < 10; i ++)
        add += parseInt(cpf.charAt(i)) * (11 - i);
    rev = 11 - (add % 11);
    if (rev == 10 || rev == 11)
        rev = 0;
    if (rev != parseInt(cpf.charAt(10)))
        return false;
    return true;
};
User.EmailRegex = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
User.prototype.isEmailValid = function(){
    return User.EmailRegex.test(this.Email);
};
User.prototype.age = function(){
    if (!this.Birthday) return 0;

    var dt = new Date();
    var bd = this.Birthday;

    var age = dt.getFullYear() - bd.getFullYear() - 1;

    if (dt.getDate() >= bd.getDate() && dt.getMonth() === bd.getMonth()){
        age++;
    }else if (dt.getMonth() > bd.getMonth()){
        age++
    }

    return age;
};
User.prototype.hasRole = function(role){
    return this.Role & role;
};

User.Genders = {
    Male: 1,
    Female: 2,
    NB: 3,
    Undisclosed: 4,
    _strings: {
        1: 'Masculino',
        2: 'Feminino',
        3: 'Não binário',
        4: 'Não informado'
    }
};

User.Roles = {
    Member: 1,
    Operator: 2,
    Admin: 4,
    _strings: {
        1: 'Sócio',
        2: 'Operador',
        4: 'Administrador'
    },
    _min: 1
};