<h2>Usuários</h2>

<h4>Pendendo aprovação</h4>
<div id="approval-pending">
	<paged-table :items="users" ref="table">
		<template slot="head">
			<tr>
				<th style="width:50px;">Id</th>
				<th style="width:20%;">Nome completo</th>
				<th style="width:20%;">Nome de exibição</th>
				<th>Email</th>
				<th style="width:20%;">Cidade/Estado</th>
				<th style="width:75px;">Ver...</th>
			</tr>
		</template>
		<template slot="item" scope="data">
			<tr>
				<td>{{data.item.Id}}</td>
				<td>{{data.item.FullName}}</td>
				<td>{{data.item.DisplayName}}</td>
				<td>{{data.item.Email}} {{data.item.Status == 1 ? '(não confirmado)' : ''}}</td>
				<td>
					<span v-if="data.item.Address">
						{{data.item.Address.City}}/{{data.item.Address.State}}
					</span>
				</td>
				<td>
					<div class="btn btn-sm btn-block btn-info" v-on:click="toggleDetails(data.index)"
					     v-popover.top.hover="'Ver dados'">
						<span class="glyphicon glyphicon-eye-open"></span>
					</div>
				</td>
			</tr>
			<tr>
				<td colspan="85" style="padding: 0; border: none;">
					<pane :ref="'userDetails'+data.index" :expanded="false">
						<div class="container-fluid" style="padding-bottom: 15px">
							<div class="row">
								<div class="col-md-3">
									<b>CPF:</b><br>
									{{data.item.CPF}}
								</div>
								<div class="col-md-3">
									<b>Telefone:</b><br>
									{{data.item.Telephone}}
								</div>
								<div class="col-md-3">
									<b>Nascimento:</b><br>
									{{data.item.Birthday ? data.item.Birthday.format('d/m/Y') : ''}}
								</div>
								<div class="col-md-3">
									<b>Gênero:</b><br>
									{{genders[data.item.Gender]}}
								</div>
							</div>
							<div class="row">
								<div class="col-md-6">
									<b>Endereço:</b>
									<div v-if="data.item.Address">
										{{data.item.Address.Street}} {{data.item.Address.Number}}, {{data.item.Address.NumberComplement}} <br>
										{{data.item.Address.Neighborhood}}, {{data.item.Address.City}} - {{data.item.Address.State}} {{data.item.Address.PostalCode}}
									</div>
								</div>
								<div class="col-md-6">
									<b>Ações:</b>
									<div class="row">
										<div class="col-xs-6">
											<div class="btn btn-block btn-success" v-on:click="approve(data.item)">
												<span class="glyphicon glyphicon-check"></span> Aprovar
											</div>
										</div>
										<div class="col-xs-6">
											<div class="btn btn-block btn-danger" v-on:click="deny(data.item)">
												<span class="glyphicon glyphicon-remove"></span> Negar
											</div>
										</div>
									</div>
								</div>
							</div>
						</div>
					</pane>
				</td>
			</tr>
		</template>
	</paged-table>
</div>

<?=ScriptDependency::Current()->IncludeScript('entities/User')?>
<?=ScriptDependency::Current()->IncludeScript('entities/BalanceOperation')?>
<script>
	var approveVM = new Vue({
		el: '#approval-pending',
		data: {
			users: [],
			resultsPerPage: 4,
			currentPage: 1,
			genders: User.Genders._strings
		},
		methods: {
			pageChange: function (p) {
				this.hideAllDetails(1);
				this.currentPage = p;
			},
			toggleDetails: function (idx) {
				this.$refs['userDetails'+idx].toggle();
			},
			hideAllDetails: function (instant) {
				if (!this.$refs.table) return;

				for(var i = 0; i < this.$refs.table.results.length; i++){
					this.$refs['userDetails'+i].hide(instant);
				}
			},
			load: function () {
				var vm = this;
				this.hideAllDetails();

				Loading.Show();
				User.Select(function (data) {
					vm.users = data;

					Loading.Hide();
				}, {
					filter: {
						Status: {
							values: [1,2]
						}
					}
				})
			},
			removeUser: function (user) {
				var idx = this.users.indexOf(user);
				if (idx >= 0){
					this.users.splice(idx,1)
				}
			},

			approve: function (user) {
				var vm = this;

				Loading.Show('Aprovando usuário...');

				Session.Do('User', 'Approve', { id: user.Id }, function (data) {
					Loading.Hide();
					if (data){
						vm.hideAllDetails();
						vm.removeUser(user);
						toast('success', 'Usuário aprovado', 'Sucesso', 1500);
					}else{
						alert('Error');
					}
				})
			},
			deny: function (user) {
				var vm = this;

				Loading.Show();
				Session.Do('User', 'Deny', { id: user.Id }, function (data) {
					Loading.Hide();
					if (data){
						vm.hideAllDetails();
						vm.removeUser(user);
						toast('success', 'Usuário negado', 'Sucesso', 1500);
					}else{
						alert('Error');
					}
				})
			}
		},
		mounted: function () {
			this.load();
		}
	});
</script>