<h2>Bens</h2>

<div id="plans">
	<entity-manager ref="manager" model="Good" :order="order">
		<template slot="filter">
            <label>Filtrar por:</label>
            <div class="row">
                <div class="col-md-4 col-sm-6 col-xs-12">
                    <label for="">Código</label>
                    <input type="text" class="form-control" v-model="filter.code" placeholder="">
                </div>
                <div class="col-md-4 col-sm-6 col-xs-12">
                    <label for="">Marca</label>
                    <input type="text" class="form-control" v-model="filter.brand" placeholder="">
                </div>
                <div class="col-md-4 col-sm-6 col-xs-12">
                    <label for="">Modelo</label>
                    <input type="text" class="form-control" v-model="filter.model" placeholder="">
                </div>
                <div class="col-md-4 col-sm-6 col-xs-12">
                    <label for="">Título</label>
                    <input type="text" class="form-control" v-model="filter.title" placeholder="">
                </div>

                <div class="col-md-4 col-sm-6 col-xs-12">
                    <label for="">Tipo</label><br>
                    <div class="btn-group">
                        <button class="btn" :class="filter.type.indexOf(type.Id) >=0 ? 'btn-primary' : 'btn-default'" v-for="type in types"
                                v-on:click="selectType(type)">{{type.Name}}</button>
                    </div>
                </div>

                <div class="col-md-4 col-sm-6 col-xs-12">
                    <label for="">Plano</label><br>
                    <div class="btn-group">
                        <button class="btn" :class="filter.plan.indexOf(plan.Id) >=0 ? 'btn-primary' : 'btn-default'" v-for="plan in plans"
                                v-on:click="selectPlan(plan)">{{plan.Name}}</button>
                    </div>
                </div>
            </div>
		</template>

        <template slot="table-head">
            <th style="width: 10%">Código</th>
            <th style="width: 10%">Tipo</th>
            <th style="width: 15%">Marca</th>
            <th style="width: 15%">Modelo</th>
            <th class="hidden-xs">Título</th>
        </template>
		<template slot="table-item" scope="data">
			<td>
				<small class="glyphicon glyphicon-eye-close pull-right" v-if="data.item.Hidden"
				       v-popover.top.hover="'Invisível'"></small>
				{{data.item.Code}}
			</td>
			<td>{{data.item.Type ? data.item.Type.Name : '-'}}</td>
			<td>{{data.item.Brand}}</td>
			<td>{{data.item.Model}}</td>
			<td class="hidden-xs">{{data.item.Title}}</td>
		</template>
		<template slot="edit" scope="data">
			<h4>Editar Bem</h4>

			<good-edit ref="ge" v-model="data.item" upload-url="/do/Good.ImageUpload" :types="types"></good-edit>

            <br>

            <h4>Histórico de reservas</h4>

            <paged-table :items="reservationHistory" :results-per-page="4">
                <template slot="head">
                    <tr>
                        <th style="width:100px;">Id</th>
                        <th style="width:20%;">Sócio</th>
                        <th class="hidden-xs" style="width:250px;">Data</th>
                        <th style="width:100px">Custo</th>
                        <th>Status</th>
                    </tr>
                </template>
                <template slot="item" scope="data">
                    <tr>
                        <td>{{data.item.Id.pad(3)}}</td>
                        <td>
                            {{data.item.Renter.DisplayName}}
                        </td>
                        <td class="hidden-xs">
                            <template v-if="data.item.StartDateTime.getTime() == data.item.EndDateTime.getTime()">
                                {{data.item.StartDateTime.format('d/m/Y - H:i:s')}}
                            </template>
                            <template v-else-if="data.item.StartDateTime.format('d/m/Y') == data.item.EndDateTime.format('d/m/Y')">
                                {{data.item.StartDateTime.format('d/m/Y')}} <br>
                                {{data.item.StartDateTime.format('H:i:s')}} ~
                                {{data.item.EndDateTime.format('H:i:s')}}
                            </template>
                            <template v-else>
                                De: {{data.item.StartDateTime.format('d/m/Y - H:i:s')}} <br>
                                Até: {{data.item.EndDateTime.format('d/m/Y - H:i:s')}}
                            </template>
                        </td>
                        <td>
                            {{data.item.getCost()}} pontos
                        </td>
                        <td :class="'text-'+status._classes[data.item.Status]">
                            {{status._strings[data.item.Status]}}
                        </td>
                    </tr>
                </template>
            </paged-table>


		</template>
	</entity-manager>
</div>


<div class="row">
    <div class="col-md-12">
        <br>
        <a href="/do/CSV.Goods" class="btn btn-success pull-right">
            Baixar todos em CSV <span class="glyphicon glyphicon-download"></span>
        </a>
    </div>
</div>

<?=ScriptDependency::Current()->IncludeScript('entities/Good')?>
<?=ScriptDependency::Current()->IncludeScript('entities/Reservation')?>

<?=VueParser::Inline('EntityManager')?>
<?=VueParser::Inline('FileUploader')?>
<?=VueParser::Inline('entity/GoodEdit')?>

<script type="application/json" id="types-json"><?=json_encode(BaseModel::SerializeArray(GoodType::Select()))?></script>
<script type="application/json" id="plans-json"><?=json_encode(BaseModel::SerializeArray(Plan::Select(['Active'=>1], '`Index` ASC')))?></script>

<script>
	var goodTypes = InlineJsonToEntityList('types');
    var plans = InlineJsonToEntityList('plans');


	var plansVM = new Vue({
		el: '#plans',
		data: {
			order: 'TypeId ASC',
			measurements: Good.UsageMeasurement,
			uploads: [],
			tempUrl: '',
            types: goodTypes,
            plans: plans,

            filter: {
                code: "",
                brand: "",
                model: "",
                title: "",
                type: [],
                plan: []
            },



            status: Reservation.Status,
            reservationHistory: []
        },
		computed: {
		},
		methods: {
            selectType: function (t) {
                if (this.filter.type.indexOf(t.Id)>=0){
                    var idx = this.filter.type.indexOf(t.Id);
                    this.filter.type.splice(idx, 1);
                }else{
                    this.filter.type.push(t.Id)
                }
            },
            selectPlan: function (p) {
                if (this.filter.plan.indexOf(p.Id)>=0){
                    var idx = this.filter.plan.indexOf(p.Id);
                    this.filter.plan.splice(idx, 1);
                }else{
                    this.filter.plan.push(p.Id)
                }
            }
		},
		mounted: function () {
            var vm = this;

            var filter = this.filter;

		    this.$refs.manager.filterResults(function (e) {
                return (!filter.code || e.Code.toLowerCase().indexOf(filter.code.toLowerCase()) >= 0) &&
                    (!filter.brand || e.Brand.toLowerCase().indexOf(filter.brand.toLowerCase()) >= 0) &&
                    (!filter.model || e.Model.toLowerCase().indexOf(filter.model.toLowerCase()) >= 0) &&
                    (!filter.title || e.Title.toLowerCase().indexOf(filter.title.toLowerCase()) >= 0) &&
                    (!filter.type.length || filter.type.indexOf(e.Type.Id) >= 0) &&
                    (!filter.plan.length || filter.plan.filter(function (planId) {return e.Type.PlansAllowed.indexOf(planId) >=0 }).length > 0);
            });

            this.$refs.manager.$on('edit', function (item) {
                vm.reservationHistory = [];

                Reservation.Select(function (data) {
                    vm.reservationHistory = data;
                }, {
                    filter: {
                        GoodId: item.Id
                    },
                    order: 'StartDateTime DESC'
                });
            })
		}
	});
</script>