/*
 * Description: A script
 * Version: 0.1
 * Author: Daniel Andrade
 * Date: 27/01/2017
 * This code may not be reused without proper permission from its creator.
 */
var _bootsvuePopover = function (el, binding) {
    var val = binding.value;
    var modifiers = binding.modifiers;
    var jqEl = $(el);

    if (val){
        var opts = {
            container: modifiers.body ? 'body' : false,
            trigger: modifiers.hover ? 'hover' : (modifiers.focus ? 'focus' : 'click'),
            content: val,
            html: modifiers.html ? true : false,
            placement: modifiers.top ? 'top' : (modifiers.left ? 'left' : (modifiers.bottom ? 'bottom' : 'right'))
        };

        var oldOpts = jqEl.data('vue-popover');

        if (JSON.stringify(oldOpts) == JSON.stringify(opts)) return;

        jqEl.popover('destroy');
        jqEl.popover(opts);
        jqEl.data('vue-popover', opts);

        if (modifiers.focus && el === document.activeElement){
            jqEl.popover('show');
        }
    }else{
        jqEl.popover('destroy');
        jqEl.data('vue-popover', '');
    }
};
Vue.directive('popover', {
    bind: _bootsvuePopover,
    update: _bootsvuePopover,
    unbind: function (el, binding) {
        var jqEl = $(el);
        jqEl.popover('destroy');
        jqEl.data('vue-popover', '');
    }
});

Vue.component('date-input', {
    template: '<input type="date" ref="input" v-on:input="updateValue($event.target.value)">',
    data: function () {
        return {
            currentValue: null
        }
    },
    props: [
        'value'
    ],
    watch: {
        value: function (newVal) {
            if (newVal && (!this.currentValue || newVal.getTime() != this.currentValue.getTime())){
                this.currentValue = new Date(newVal.getTime());
                this.updateInput();
            }else if (!newVal){
                this.currentValue = null;
                this.updateInput();
            }
        }
    },
    methods: {
        updateValue: function (value) {
            value = value.split('-');

            var year = parseInt(value[0]);
            var month = parseInt(value[1])-1;
            var day = parseInt(value[2]);

            if (year > 1900 && month >= 0 && day > 0){
                if (!this.currentValue){
                    this.currentValue = new Date();
                    this.currentValue.setHours(0);
                    this.currentValue.setMinutes(0);
                    this.currentValue.setSeconds(0);
                    this.currentValue.setMilliseconds(0);
                }

                this.currentValue.setYear(year);
                this.currentValue.setMonth(month);
                this.currentValue.setDate(day);

                this.$emit('input', new Date(this.currentValue.getTime()))
            }else{
                this.currentValue = null;

                this.$emit('input', null)
            }
        },
        updateInput: function () {
            if (this.currentValue){
                this.$refs.input.value = this.currentValue.format('Y-m-d');
            }else{
                this.$refs.input.value = '';
            }
        }
    },
    mounted: function () {
        if (this.value){
            this.currentValue = this.value;
            this.updateInput();
        }
    }
});

Vue.component('time-input', {
    template: '<input type="time" ref="input" :disabled="!currentValue" v-on:input="updateValue($event.target.value)">',
    data: function () {
        return {
            currentValue: null
        }
    },
    props: [
        'value'
    ],
    watch: {
        value: function (newVal) {
            if (newVal && (!this.currentValue || newVal.getTime() != this.currentValue.getTime())){
                this.currentValue = new Date(newVal.getTime());
                this.updateInput();
            }else if (!newVal){
                this.currentValue = null;
                this.updateInput();
            }
        }
    },
    methods: {
        updateValue: function (value) {
            value = value.split(':');

            var hour = parseInt(value[0]);
            var minute = parseInt(value[1]);

            if (!isNaN(hour) && !isNaN(minute)){
                this.currentValue.setHours(hour);
                this.currentValue.setMinutes(minute);

                this.$emit('input', new Date(this.currentValue.getTime()))
            }
        },
        updateInput: function () {
            if (this.currentValue){
                this.$refs.input.value = this.currentValue.format('H:i');
            }else{
                this.$refs.input.value = '';
            }
        }
    },
    mounted: function () {
        if (this.value){
            this.currentValue = this.value;
            this.updateInput();
        }
    }
});


Vue.component('cpf-input', {
    template: '<input type="tel" ref="input" v-on:input="updateValue($event.target.value)" v-popover.focus.top="popover">',
    data: function () {
        return {
            currentValue: null
        }
    },
    props: [
        'value'
    ],
    watch: {
        value: function (newVal) {
            if (newVal != this.currentValue){
                this.currentValue = newVal;
                this.updateInput();
            }
        }
    },
    computed: {
        popover: function () {
            if (this.valid){
                return '';
            }else{
                if (!this.currentValue){
                    return 'xxx.xxx.xxx-xx'
                }else{
                    return 'CPF inválido'
                }
            }
        },
        valid: function () {
            if (!this.currentValue) return null;

            var cpf = this.currentValue.replace(/[^\d]+/g,'');

            if(cpf == '') return false;
            // Elimina CPFs invalidos conhecidos
            if (cpf.length != 11 ||
                cpf == "00000000000" ||
                cpf == "11111111111" ||
                cpf == "22222222222" ||
                cpf == "33333333333" ||
                cpf == "44444444444" ||
                cpf == "55555555555" ||
                cpf == "66666666666" ||
                cpf == "77777777777" ||
                cpf == "88888888888" ||
                cpf == "99999999999")
                return false;
            // Valida 1o digito
            var add = 0;
            for (var i=0; i < 9; i ++)
                add += parseInt(cpf.charAt(i)) * (10 - i);
            var rev = 11 - (add % 11);
            if (rev == 10 || rev == 11)
                rev = 0;
            if (rev != parseInt(cpf.charAt(9)))
                return false;
            // Valida 2o digito
            add = 0;
            for (var i = 0; i < 10; i ++)
                add += parseInt(cpf.charAt(i)) * (11 - i);
            rev = 11 - (add % 11);
            if (rev == 10 || rev == 11)
                rev = 0;
            if (rev != parseInt(cpf.charAt(10)))
                return false;

            return true;
        }
    },
    methods: {
        updateValue: function (value) {
            var str = value.replace(/[^0-9]/gi, "");

            if (str.length > 11){
                str = str.substr(0,11);
            }

            if (str.length != 11){
                this.currentValue = '';

                this.$emit('input', this.currentValue);
            }else{
                str = str.substr(0,3) + "." + str.substr(3,3) + "." + str.substr(6,3) + "-" + str.substr(9,2);

                if (this.currentValue != str){
                    this.currentValue = str;
                }

                this.updateInput();

                if (this.valid){
                    this.$emit('input', this.currentValue);
                }else{
                    this.$emit('input', '');
                }
            }
        },
        updateInput: function () {
            if (this.currentValue){
                this.$refs.input.value = this.currentValue;
            }else{
                this.$refs.input.value = '';
            }
        }
    },
    mounted: function () {
        if (this.value){
            this.currentValue = this.value;

            this.updateInput();
        }
    }
});


Vue.component('telephone-input', {
    template: '<input type="tel" ref="input" v-on:input="updateValue($event.target.value)" v-popover.focus.top="popover">',
    data: function () {
        return {
            currentValue: null
        }
    },
    props: [
        'value'
    ],
    computed: {
        popover: function () {
            if (this.currentValue){
                return '';
            }else{
                return 'Lembre de incluir o DDD, o DDI é opcional.'
            }
        }
    },
    watch: {
        value: function (newVal) {
            if (newVal != this.currentValue){
                this.currentValue = newVal;
                this.updateInput();
            }
        }
    },
    methods: {
        updateValue: function (value) {
            var str = value.replace(/[^0-9]/gi, "");

            if (str.length == 14){
                // 999 99 99999-9999
                str = "+" + str.substr(0,3) + " " + str.substr(3,2) + " " + str.substr(5,5) + "-" + str.substr(10,4);

            }else if (str.length == 13){
                // 99 99 99999-9999
                str = "+" + str.substr(0,2) + " " + str.substr(2,2) + " " + str.substr(4,5) + "-" + str.substr(9,4);

            }else if (str.length == 12){
                // 99 99 9999-9999
                str = "+" + str.substr(0,2) + " " + str.substr(2,2) + " " + str.substr(4,4) + "-" + str.substr(8,4);

            }else if (str.length == 11){
                // 99 99999-9999
                str = str.substr(0,2) + " " + str.substr(2,5) + "-" + str.substr(7,4);

            }else if (str.length == 10){
                // 99 9999-9999
                str = str.substr(0,2) + " " + str.substr(2,4) + "-" + str.substr(6,4);
            }else{
                str = '';
            }

            this.currentValue = str;

            if (this.currentValue){
                this.updateInput();
            }

            this.$emit('input', this.currentValue);
        },
        updateInput: function () {
            if (this.currentValue){
                this.$refs.input.value = this.currentValue;
            }else{
                this.$refs.input.value = '';
            }
        }
    },
    mounted: function () {
        if (this.value){
            this.currentValue = this.value;

            this.updateInput();
        }
    }
});


Vue.component('postalcode-input', {
    template: '<input type="tel" ref="input" v-on:input="updateValue($event.target.value)" maxlength="9">',
    data: function () {
        return {
            currentValue: null
        }
    },
    props: [
        'value'
    ],
    computed: {
    },
    watch: {
        value: function (newVal) {
            if (newVal != this.currentValue){
                this.currentValue = newVal;
                this.updateInput();
            }
        }
    },
    methods: {
        updateValue: function (value) {
            var str = value.replace(/[^0-9]/gi, "");

            if (str.length == 8){
                // 999 99 99999-9999
                str = str.substr(0,5) + "-" + str.substr(5,3);
            }

            this.currentValue = str;

            if (this.currentValue){
                this.updateInput();
            }

            this.$emit('input', this.currentValue);
        },
        updateInput: function () {
            if (this.currentValue){
                this.$refs.input.value = this.currentValue;
            }else{
                this.$refs.input.value = '';
            }
        }
    },
    mounted: function () {
        if (this.value){
            this.currentValue = this.value;

            this.updateInput();
        }
    }
});


Vue.component('place-input', {
    template: '<input type="text" ref="input" placeholder="Escolha um local...">',
    data: function () {
        return {
            placesSearch: null,
            place: null
        }
    },
    props: [
        'value'
    ],
    computed: {
    },
    watch: {
        value: function (newVal) {
            this.updateInput(newVal);
        }
    },
    methods: {
        updateValue: function (value) {
            this.$emit('input', value);
        },
        updateInput: function (val) {
            if (!this.place || val != this.place.formatted_address){
                this.$refs.input.value = val;

                this.place = null;
            }
        }
    },
    mounted: function () {
        if (this.value)
            this.updateInput(this.value);

        var vm = this;

        var input = this.$refs.input;

        GoogleMaps.OnLoad(function () {
            vm.placesSearch = new google.maps.places.Autocomplete(input,{
                //types: ['address']
            });

            vm.placesSearch.addListener('place_changed', function() {
                var place = vm.placesSearch.getPlace();

                if (!place.geometry) {
                    vm.place = null;
                    vm.updateValue('');
                }else{
                    vm.place = place;
                    vm.updateValue(place.formatted_address);
                }
            });
        });
    }
});